/* eslint-disable @typescript-eslint/no-explicit-any */
export interface PipeTarget {
  addEventListener: (m: "message", f: (ev: MessageEvent) => any) => void;
  postMessage(message: any): void;
}

export type RetType<T> = T extends (...args: any[]) => infer R ? R : never;
export type ArgsType<T> = T extends (...args: infer A) => any ? A : never;
export type MethodType<T> = T extends (...args: any[]) => any ? T : never;

export default function createPipe<T>(target: PipeTarget) {
  let closed = false;
  const handlers = new Map<string, (...args: any[]) => any>();
  const callbacks = new Map<string, (result: any) => void>();

  target.addEventListener("message", (event) => {
    const { type } = event.data;
    if (type === "call") {
      const { id, method, args } = event.data;
      const handler = handlers.get(method);
      if (handler) {
        try {
          target.postMessage({ type: "result", id, result: handler(...args) });
        } catch (e) {
          console.log(e);
        }
      }
    }
    if (type === "result") {
      const { id, result } = event.data;
      const callback = callbacks.get(id);
      if (callback) {
        callback(result);
      }
    }
  });

  return {
    call<K extends keyof T>(method: K, ...args: ArgsType<T[K]>) {
      return new Promise<RetType<T[K]>>((resolve, reject) => {
        if (closed) {
          return reject(new Error("Pipe is closed"));
        }

        const id = Math.random().toString(36).slice(2);
        callbacks.set(id, (result) => {
          resolve(result);
          callbacks.delete(id);
        });

        setTimeout(() => {
          if (callbacks.has(id)) {
            callbacks.delete(id);
            reject(new Error("Timeout"));
          }
        }, 500);
        target.postMessage({ type: "call", id, method, args });
      });
    },

    on<K extends keyof T & string>(method: K, handler: MethodType<T[K]>) {
      if (closed) {
        throw new Error("Pipe is closed");
      }
      handlers.set(method, handler);
    },
    close() {
      closed = true;
    },
  };
}