import nunjucks from "nunjucks";
import createPipe from "./pipe";
import type { Rpc } from "./types";
import defaultTemplate from "../layouts/bubbles";

// I've heard that nunjucks is vulnerable to SSTI
// But I think this is enough to prevent it
const removeConstructor = (obj: any) =>
  Object.defineProperty(obj.__proto__, "constructor", { value: null });

removeConstructor(async function* () {});
removeConstructor(async function () {});
removeConstructor(function* () {});
removeConstructor(function () {});
// ----------------------------

nunjucks.configure({ autoescape: true, noCache: true });

const pipe = createPipe<Rpc>(self);
const messages: {
  author: string;
  content: string;
  time: Date;
}[] = [];
const userTemplates = new Map<string, string>();

pipe.on("addMessage", (userId, content) => {
  messages.push({
    author: userId,
    content,
    time: new Date(),
  });
});

pipe.on("render", (userId) => {
  const msgs = messages.map((msg) => ({...msg, author: msg.author === userId}))
  return nunjucks.renderString(userTemplates.get(userId) ?? defaultTemplate, {
    messages: msgs,
    userId,
    split: (s: string, size = 32) => {
      const result = [];
      for (let i = 0; i < s.length; i += size) {
        result.push(s.slice(i, i + size));
      }
      return result.join(" ");
    },
  });
});

pipe.on("updateTemplate", (userId, template) => {
  userTemplates.set(userId, template);
});
