import createPipe from "./pipe";
import type { Rpc } from "./types";

const WORKER = new URL("../../server/worker.ts", import.meta.url).href;
const workers = new Map<string, RenderingWorker>();

type RenderingWorker = Awaited<ReturnType<typeof createWorker>>;

function createAlarm(keepAlive: number, callback: () => void) {
  let lastPing = Date.now();

  function handleAlarm() {
    const now = Date.now();
    if (now - lastPing > keepAlive) {
      callback();
    } else {
      setTimeout(handleAlarm, keepAlive - (now - lastPing));
    }
  }

  setTimeout(handleAlarm, keepAlive);

  return () => (lastPing = Date.now());
}

async function createWorker(id: string) {
  const worker = new Worker(WORKER, { type: "module", smol: true } as WorkerOptions);
  const pipe = createPipe<Rpc>(worker);
  const stop = () => {
    workers.delete(id);
    pipe.close();
    worker.terminate();
  };
  const keepAlive = createAlarm(10 * 1000, stop);
  return {
    pipe,
    id,
    stop,
    keepAlive,
  };
}

export async function getWorker(id: string) {
  let worker = workers.get(id);
  if (!worker) {
    worker = await createWorker(id);
    workers.set(id, worker);
  }
  worker.keepAlive();
  return worker;
}
