import puppeteer from "puppeteer";

const sleep = (ms: number) => new Promise((r) => setTimeout(r, ms));

const FLAG = "FCSC{flag_placeholder}";

const adminMessages = [
  "Hello, I'm the admin of this chat. I'm here to help you if you have any questions.",
  "I'm sorry, I can't help you with that.",
  "You can have this flag if you want: " + FLAG,
];

async function visit(roomId: string) {
  let browser;
  try {
    browser = await puppeteer.launch({
      headless: true,
      executablePath: "/usr/bin/chromium-browser",
      args: [
        "--incognito",
        "--no-sandbox",
        "--disable-setuid-sandbox",
        "--disable-gpu",
        "--disable-jit",
        "--disable-wasm",
        "--disable-dev-shm-usage",
        "--ignore-certificate-errors",
        "--js-flags=--noexpose_wasm,--jitless",
      ],
    });

    const page = await browser.newPage();

    await page.goto(`http://127.0.0.1:3000/chat/${roomId}`);
    await page.waitForSelector("#loaded", {
      timeout: 3000,
    });

    for (const message of adminMessages) {
      await page.focus("#form-input");
      await page.keyboard.type(message);
      await sleep(200);
      await page.click("#form-btn");
      await sleep(2000);
    }
  } finally {
    await browser?.close();
  }
}

async function run(roomId: string | null) {
  if (!roomId?.match(/^[a-f0-9]{32}$/)) {
    return "Invalid room id!";
  }
  try {
    await visit(roomId);
    return `Visited room ${roomId}!`;
  } catch (err) {
    return "Something went wrong!";
  }
}

Bun.serve({
  port: 4242,
  hostname: "127.0.0.1",
  async fetch(req) {
    const url = new URL(req.url);
    const result = await run(url.searchParams.get("roomId"));
    return new Response(result);
  },
});
